{% extends "otree/Session.html" %}



{% block content %}
    {{ super() }}
    <form action="" method="get" role="form" class="form" id="form">
        <table cellpadding="10">
            <tr>
                <td>{% formfield 'app_name' %}</td>
                <td>{% formfield 'round_number' %}</td>
                <td>
                    <button type="submit" id="submit" class="btn btn-primary">
                        Go
                    </button>
                </td>
            </tr>
        </table>
    </form>

    {% include user_template %}

    {% if is_debug %}
        <br><br>
        {% include 'otree/includes/debug_info.html' %}
    {% endif %}

    <script>
    $(function() {
        $('#id_app_name').change(function() {
            $('#id_round_number').val('');
        });
    });
    </script>

{% endblock %}
