


<div class="otree-timer alert alert-warning">
    <p>{{ timer_text }}
        <span style="font-weight: bold">
            <span class="otree-timer__time-left"></span>
        </span>
    </p>
</div>
